using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER UTF8String object.
	*/
	public class DERUTF8String : DERObject, DERString
	{
		String  m_string;

		/**
		* return an UTF8 string from the passed in object.
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERUTF8String getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERUTF8String).IsInstanceOfType(obj)))
			{
				return (DERUTF8String)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERUTF8String(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an UTF8 String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERUTF8String getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - byte encoded string.
		*/
		internal DERUTF8String(
			byte[]   in_string)
		{
			int i = 0;
			int length = 0;

			while (i < in_string.Length)
			{
				length++;
				if ((in_string[i] & 0xe0) == 0xe0)
				{
					i += 3;
				}
				else if ((in_string[i] & 0xc0) == 0xc0)
				{
					i += 2;
				}
				else
				{
					i += 1;
				}
			}

			char[]  cs = new char[length];

			i = 0;
			length = 0;

			while (i < in_string.Length)
			{
				char    ch;

				if ((in_string[i] & 0xe0) == 0xe0)
				{
					ch = (char)(((in_string[i] & 0x1f) << 12)
						| ((in_string[i + 1] & 0x3f) << 6) | (in_string[i + 2] & 0x3f));
					i += 3;
				}
				else if ((in_string[i] & 0xc0) == 0xc0)
				{
					ch = (char)(((in_string[i] & 0x3f) << 6) | (in_string[i + 1] & 0x3f));
					i += 2;
				}
				else
				{
					ch = (char)(in_string[i] & 0xff);
					i += 1;
				}

				cs[length++] = ch;
			}

			this.m_string = new String(cs);
		}

		/**
		* basic constructor
		*/
		public DERUTF8String(
			String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public override int GetHashCode()
		{
			return this.getString().GetHashCode();
		}

		public override bool Equals(Object  o)
		{
			if (!(typeof(DERUTF8String).IsInstanceOfType(o)))
			{
				return false;
			}

			DERUTF8String  s = (DERUTF8String)o;

			return this.getString().Equals(s.getString());
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			char[]                  c = m_string.ToCharArray();
			MemoryStream   bOut = new MemoryStream();

			for (int i = 0; i != c.Length; i++)
			{
				char    ch = c[i];

				if (ch < 0x0080)
				{
					bOut.WriteByte((byte)(ch));
				}
				else if (ch < 0x0800)
				{
					bOut.WriteByte((byte)(0xc0 | (ch >> 6)));
					bOut.WriteByte((byte)(0x80 | (ch & 0x3f)));
				}
				else
				{
					bOut.WriteByte((byte)(0xe0 | (ch >> 12)));
					bOut.WriteByte((byte)(0x80 | ((ch >> 6) & 0x3F)));
					bOut.WriteByte((byte)(0x80 | (ch & 0x3F)));
				}
			}

			outStream.writeEncoded(DERTags.UTF8_STRING, bOut.ToArray());
		}
	}

}